@@ -1,7 +1,7 @@
 /***************************************************************************/
 /*									   */
 /* ftplib.c - callable ftp access routines				   */
-/* Copyright (C) 1996-2001, 2013 Thomas Pfau, tfpfau@gmail.com		   */
+/* Copyright (C) 1996-2001, 2013, 2016 Thomas Pfau, tfpfau@gmail.com	   */
 /*	1407 Thomas Ave, North Brunswick, NJ, 08902			   */
 /*									   */
 /* This library is free software.  You can redistribute it and/or	   */
@@ -488,7 +488,8 @@ GLOBALDEF int FtpConnect(const char *host, netbuf **nControl)
 	struct hostent he;
 	char tmpbuf[TMP_BUFSIZ];
 	int i, herr;
-	if ( ( i = gethostbyname_r( lhost, &he, tmpbuf, TMP_BUFSIZ, &phe, &herr ) ) != 0 )
+	if ( ( ( i = gethostbyname_r( lhost, &he, tmpbuf, TMP_BUFSIZ, &phe, &herr ) ) != 0 ) ||
+	     ( phe == NULL ) )
 	{
 	    if ( ftplib_debug )
 		fprintf(stderr, "gethostbyname: %s\n", hstrerror(herr));
@@ -549,6 +550,7 @@ GLOBALDEF int FtpConnect(const char *host, netbuf **nControl)
     ctrl->handle = sControl;
     ctrl->dir = FTPLIB_CONTROL;
     ctrl->ctrl = NULL;
+    ctrl->data = NULL;
     ctrl->cmode = FTPLIB_DEFMODE;
     ctrl->idlecb = NULL;
     ctrl->idletime.tv_sec = ctrl->idletime.tv_usec = 0;
@@ -820,10 +822,12 @@ static int FtpOpenPort(netbuf *nControl, netbuf **nData, int mode, int dir)
     ctrl->xfered = 0;
     ctrl->xfered1 = 0;
     ctrl->cbbytes = nControl->cbbytes;
+    ctrl->ctrl = nControl;
     if (ctrl->idletime.tv_sec || ctrl->idletime.tv_usec || ctrl->cbbytes)
 	ctrl->idlecb = nControl->idlecb;
     else
 	ctrl->idlecb = NULL;
+    nControl->data = ctrl;
     *nData = ctrl;
     return 1;
 }
@@ -957,8 +961,6 @@ GLOBALDEF int FtpAccess(const char *path, int typ, int mode, netbuf *nControl,
 	*nData = NULL;
 	return 0;
     }
-    (*nData)->ctrl = nControl;
-    nControl->data = *nData;
     if (nControl->cmode == FTPLIB_PORT)
     {
 	if (!FtpAcceptConnection(*nData,nControl))
@@ -1055,7 +1057,7 @@ GLOBALDEF int FtpClose(netbuf *nData)
 	ctrl = nData->ctrl;
 	free(nData);
 	ctrl->data = NULL;
-	if (ctrl && ctrl->response[0] != '4' && ctrl->response[0] != 5)
+	if (ctrl && ctrl->response[0] != '4' && ctrl->response[0] != '5')
 	{
 	    return(readresp('2', ctrl));
 	}
@@ -1237,7 +1239,7 @@ static int FtpXfer(const char *localfile, const char *path,
 	if (localfile)
 	{
 	    fclose(local);
-	    if ( typ == FTPLIB_FILE_WRITE )
+	    if ( typ == FTPLIB_FILE_READ )
 		unlink(localfile);
 	}
 	return 0;
